"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _postcss = _interopRequireDefault(require("postcss"));

var _postcssSelectorParser = _interopRequireDefault(require("postcss-selector-parser"));

var _postcssValueParser = _interopRequireDefault(require("postcss-value-parser"));

var _package = require("../package.json");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const defaults = {
  edgeAutohide: false
};
const widthMap = {
  auto: 'initial',
  thin: '0.5rem',
  none: '0'
};
const colorMap = {
  auto: 'initial',
  dark: 'initial',
  light: 'initial'
};

var _default = _postcss.default.plugin(_package.name, (options = defaults) => (css, result) => {
  css.walkDecls(/^scrollbar/, decl => {
    if (decl.prop === 'scrollbar-width') {
      return processWidth(decl);
    }

    if (decl.prop === 'scrollbar-color') {
      return processColor(decl);
    }
  });

  function processWidth(decl) {
    let {
      parent,
      value: keyword
    } = decl;
    let root = parent.parent;

    if (!/auto|thin|none/.test(keyword)) {
      return decl.warn(result, 'Invalid value for property `scrollbar-width`. ' + 'Must be one of `auto | thin | none`.', {
        word: keyword
      });
    }

    let processor = (0, _postcssSelectorParser.default)(selectors => {
      selectors.each(selector => {
        selector.append(_postcssSelectorParser.default.pseudo({
          value: '::-webkit-scrollbar'
        }));
      });
    });

    let newRule = _postcss.default.rule({
      selector: processor.processSync(parent.selector)
    }).append(_postcss.default.decl({
      prop: 'width',
      value: widthMap[keyword]
    }));

    root.insertBefore(parent, newRule);
    let value = options.edgeAutohide ? '-ms-autohiding-scrollbar' : keyword === 'none' ? 'none' : 'auto';
    parent.insertBefore(decl, {
      prop: '-ms-overflow-style',
      value
    });
  }

  function processColor(decl) {
    let {
      nodes
    } = (0, _postcssValueParser.default)(decl.value);

    if (nodes.length === 1 && !/auto|dark|light/.test(nodes[0].value)) {
      return decl.warn(result, 'Invalid value for property `scrollbar-color`. ' + 'Must be one of `auto | dark | light | <color> <color>`.', {
        word: nodes[0].value
      });
    }

    let values = nodes.filter(value => value.type === 'word').reduce((acc, curr, idx) => {
      if (idx >= 1) {
        return _objectSpread({}, acc, {
          track: colorMap[curr.value] || curr.value
        });
      }

      return {
        thumb: colorMap[curr.value] || curr.value,
        track: colorMap[curr.value] || curr.value
      };
    }, {});
    let {
      parent
    } = decl;
    let root = parent.parent;
    Object.keys(values).forEach(pseudo => {
      let processor = (0, _postcssSelectorParser.default)(selectors => {
        selectors.each(selector => {
          selector.append(_postcssSelectorParser.default.pseudo({
            value: `::-webkit-scrollbar-${pseudo}`
          }));
        });
      });

      let newRule = _postcss.default.rule({
        selector: processor.processSync(parent.selector)
      }).append(_postcss.default.decl({
        prop: 'background-color',
        value: values[pseudo]
      }));

      root.insertBefore(parent, newRule);
    });
  }
});

exports.default = _default;